#!/usr/bin/env python3

'''
face detection using haar cascades

USAGE:
    facedetect.py [--cascade <cascade_fn>] [--nested-cascade <cascade_fn>] [<video_source>]
'''

# Python 2/3 compatibility
from __future__ import print_function

import numpy as np
import cv2
import subprocess
import os
import time


def detect(img, cascade):
    rects = cascade.detectMultiScale(img, scaleFactor=4, minNeighbors=3, minSize=(25, 25),
                                     flags=cv2.CASCADE_SCALE_IMAGE)
    if len(rects) == 0:
        return []
    rects[:,2:] += rects[:,:2]
    return rects

def draw_rects(img, rects, color):
    for x1, y1, x2, y2 in rects:
        cv2.rectangle(img, (x1, y1), (x2, y2), color, 2)

def main():
    import sys, getopt

    prev_frame_time = 0
    new_frame_time = 0
    fps_average_count = 0
    fps = 0
    font = cv2.FONT_HERSHEY_SIMPLEX
    TopLeftCornerOfText = (10,30)
    fontScale           = 1
    fontColor           = (255,255,255)
    lineType            = 2

    args, video_src = getopt.getopt(sys.argv[1:], '', ['cascade=', 'nested-cascade='])
    try:
        video_src = video_src[0]
    except:
        video_src = 0
    args = dict(args)
    cascade_fn = args.get('--cascade', "haarcascade_frontalface_default.xml")
#    cascade_fn = args.get('--cascade', "haarcascade_frontalface_alt.xml")
    nested_fn  = args.get('--nested-cascade', "haarcascade_eye.xml")

    cascade = cv2.CascadeClassifier(cv2.samples.findFile(cascade_fn))
    nested = cv2.CascadeClassifier(cv2.samples.findFile(nested_fn))

    print ("OpenCV version :  {0}".format(cv2.__version__))

    # read camera parameter at CSI1/CAM1 via ISI
    if os.path.islink("/dev/video-isi-csi1"):
        CSI1_DEVICE = "/dev/video-isi-csi1"
    else:
        print("No ISI interface on CSI1 configured")
        return 1


    print (" capture device for CSI1 port =", CSI1_DEVICE)

    if os.path.islink("/dev/cam-csi1"):
        CAM1_DEVICE = "/dev/cam-csi1"
    elif os.path.islink("/dev/cam-csi1-port0"):
        CAM1_DEVICE = "/dev/cam-csi1-port0"
    else:
        print("No camera found on CSI1 interface")
        return 1

    print (" camera device for CSI1 port  =", CAM1_DEVICE)

    CAM_TYP = ("cat /sys/class/video4linux/%s/name" % os.readlink( CAM1_DEVICE ))
    CAM_TYP = os.popen(str(CAM_TYP)).read()[:-1]				# cut the "\n" at the end
    print (" Camera Typ =", CAM_TYP)
    CAM_DRIVER = ( "echo %s | cut -d\" \" -f1" % CAM_TYP)
    CAM_DRIVER = os.popen(str(CAM_DRIVER)).read()[:-1]			# cut the "\n" at the end
    print (" Camera Driver Name =", CAM_DRIVER)

    COLOR_FMT = f"v4l2-ctl -d {CAM1_DEVICE} --get-subdev-fmt 0 | grep \"Mediabus Code\" | sed \'s/.*BUS_FMT_\\([A-Z]*\).*/\\1/g\'"
    COLOR_FMT = os.popen(str(COLOR_FMT)).read()[:-1]			# cut the "\n" at the end
    print (" Color Format =", COLOR_FMT)


    if CAM_DRIVER == "ar0144" and COLOR_FMT == "Y":
        v4l2_ctl_param = [
            "auto_exposure=0",
            "autogain_analogue=1",
            "auto_exposure_max=720",
            "dynamic_defect_pixel_correction=1"
        ]
        CAM_COL_FMT = "Y8_1X8"
        SENSOR_RES = "1280x720"
        OFFSET_SENSOR = "(0,44)"
    elif CAM_DRIVER == "ar0144" and COLOR_FMT == "SGRBG":
        v4l2_ctl_param = [
            "auto_exposure=0",
            "autogain_analogue=1",
            "auto_exposure_max=720",
            "dynamic_defect_pixel_correction=1",
            "digital_gain_red=1300",
            "digital_gain_blue=1500",
            "digital_gain_green_red=1000",
            "digital_gain_green_blue=1000"
        ]
        CAM_COL_FMT = "SGRBG8_1X8"
        SENSOR_RES = "1280x720"
        OFFSET_SENSOR = "(0,44)"
    elif CAM_DRIVER == "ar0521" and COLOR_FMT == "Y":
        v4l2_ctl_param = [
            "exposure=2240",
            "dynamic_defect_pixel_correction=1"
        ]
        CAM_COL_FMT = "Y8_1X8"
        SENSOR_RES = "1280x720"
        OFFSET_SENSOR = "(656,612)"
    elif CAM_DRIVER == "ar0521" and COLOR_FMT == "SGRBG":
        v4l2_ctl_param = [
            "exposure=2240",
            "dynamic_defect_pixel_correction=1",
            "digital_gain_red=1400",
            "digital_gain_blue=1400",
            "digital_gain_green_red=1000",
            "digital_gain_green_blue=1000"
        ]
        CAM_COL_FMT = "SGRBG8_1X8"
        SENSOR_RES = "1280x720"
        OFFSET_SENSOR = "(656,612)"
    elif CAM_DRIVER == "ar0234" and COLOR_FMT == "Y":
        v4l2_ctl_param = [
            "auto_exposure=0",
            "autogain_analogue=1",
            "auto_exposure_max=1200",
            "dynamic_defect_pixel_correction=1"
        ]
        CAM_COL_FMT = "Y8_1X8"
        SENSOR_RES = "1280x720"
        OFFSET_SENSOR = "(320,240)"
    elif CAM_DRIVER == "ar0234" and COLOR_FMT == "SGRBG":
        v4l2_ctl_param = [
            "auto_exposure=0",
            "autogain_analogue=1",
            "auto_exposure_max=1200",
            "dynamic_defect_pixel_correction=1",
            "digital_gain_red=1300",
            "digital_gain_blue=1500",
            "digital_gain_green_red=1000",
            "digital_gain_green_blue=1000"
        ]
        CAM_COL_FMT = "SGRBG8_1X8"
        SENSOR_RES = "1280x720"
        OFFSET_SENSOR = "(320,240)"
    else:
        print ("wrong number")
        sys.exit(0)

    cam_width = os.popen(str( "echo %s | cut -d\"x\" -f1" % SENSOR_RES)).read()[:-1]
    print (" cam_width =", cam_width)
    cam_height = os.popen(str( "echo %s | cut -d\"x\" -f2" % SENSOR_RES)).read()[:-1]
    print (" cam_width =", cam_height)


    # set display
    os.environ["DISPLAY"] = ':0.0'


    # set video path, camera resolution and ROI position via media-ctl
    print ("")
    print (" configure CSI1_DEVICE (camera_1 port) with media_ctl")
    print (" Note: For use other resolutions,")
    print (" see the scripts in path ...\\gstreamer-examples\\...")
    print (" ===============================================================")

    subprocess.run(["setup-pipeline-csi1", "-f", str(CAM_COL_FMT), "-s", str(SENSOR_RES), "-o", str(OFFSET_SENSOR), "-c", str(SENSOR_RES)])


    # set camera parameters via v4l2-ctl
    print(" Setting V4L2 Controls:")
    print(" ======================")
    for ctl in v4l2_ctl_param:
        print(f"   v4l2-ctl -d {CAM1_DEVICE} -c {ctl}")

    subprocess.run(["v4l2-ctl", "-d", CAM1_DEVICE, "-c", ",".join(v4l2_ctl_param)])


    # open videodevice via GStreamer
    if CAM_COL_FMT == "SGRBG8_1X8":
        cap = cv2.VideoCapture("v4l2src device="+str(CSI1_DEVICE)+" ! video/x-bayer, format=grbg, width="+str(cam_width)+", height="+str(cam_height)+" ! appsink", cv2.CAP_GSTREAMER)
    else:
        cap = cv2.VideoCapture("v4l2src device="+str(CSI1_DEVICE)+" ! video/x-raw, format=GRAY8,depth=8, width="+str(cam_width)+", height="+str(cam_height)+" ! appsink", cv2.CAP_GSTREAMER)


    if not cap.isOpened():
        print(" Cannot open camera")
        exit()


    while True:
        _ret, vis = cap.read()

        if CAM_COL_FMT == "SGRBG8_1X8":
            gray = cv2.cvtColor(vis, cv2.COLOR_BAYER_GB2GRAY)
            vis = cv2.cvtColor(vis, cv2.COLOR_BAYER_GB2BGR)
        else :
            gray = vis

        gray = cv2.equalizeHist(gray)

        rects = detect(gray, cascade)
        draw_rects(vis, rects, (0, 255, 0))
        if not nested.empty():
            for x1, y1, x2, y2 in rects:
                vis_roi = vis[y1:y2, x1:x2]
                subrects = detect(vis_roi.copy(), nested)
                draw_rects(vis_roi, subrects, (255, 0, 0))

        if fps_average_count == 5:
            new_frame_time = time.time()
            fps = 5/(new_frame_time-prev_frame_time)
            prev_frame_time = new_frame_time
            print ("fps = %3.1f"% fps, end='\r')
            fps_average_count = 0
        fps_average_count += 1
        cv2.putText(vis, "FPS: {:03.1f}".format(fps), TopLeftCornerOfText, font, fontScale, fontColor, lineType)

        cv2.imshow('facedetect', vis)

        if cv2.waitKey(5) == 27:
            break

    print('Done')


if __name__ == '__main__':
    print(__doc__)
    main()
    cv2.destroyAllWindows()
